from setuptools import setup

setup(
    author          = 'Jim Chen',
    author_email    = 'jim.sxchen@gmail.com',
    maintainer      = 'Jim Chen',
    maintainer_email= 'jim.sxchen@gmail.com',
    url             = 'https://github.com/jimsxchen/genlogger.git',
    license         = 'LICENSE',
    packages        = ['jlogger'],
    description     = 'Setup the root logger as per the handler type with formatter',
    name            = 'jlogger',
    version         = '1.1.5',
)